/// <reference types="node" />
export import URL = require("url");
/**
 * Sames are Node's url.parse() just adds the 'username', 'password' and 'scheme' fields.
 *
 * Note that `scheme` is always lower-cased (e.g. `ftp`).
 * @param url
 * @ignore
 */
export declare function parseUrl(url: string): URL.UrlWithStringQuery & {
    username?: string;
    password?: string;
    scheme?: string;
};
/** vm specific network features.   These work on google cloud, but some work on other providers */
export declare namespace vm {
    /** External IP address of the VM.   Works on google cloud, AWS, or Azure.
     *
     * solution from: https://stackoverflow.com/questions/23362887/can-you-get-external-ip-address-from-within-a-google-compute-vm-instance
    
    for example, on google you could do ```curl --header "Metadata-Flavor:Google" http://metadata/computeMetadata/v1/instance/network-interfaces/0/access-configs/0/external-ip```
    */
    function getExternalIp(): Promise<string>;
    /**
 *  extract the final client ip address, for use either when using a "bare" server or when using Google Compute's HTTP Load Balancer.
 *
 * @param request
 */
    function extractClientIpBareOrGCloudHttpLoadBalancer(
    /** works directly with hapi's request object */
    request: {
        info: {
            remoteAddress: string;
        };
        headers: Record<string, string>;
    }, forwardingSteps?: number, proxyIfRemoteAddressStartsWith?: string, ipIfInvalid?: string): string;
}
//# sourceMappingURL=net.d.ts.map